@@ -36,19 +36,16 @@
 #'
 #' @export
 #'
-#' @examples
+#' @examplesIf nrow(system_fonts()) >= 4
 #' # Create a random font collection
 #' fonts <- system_fonts()
-#' plain <- sample(which(!fonts$italic & fonts$weight <= 'normal'), 1)
-#' bold <- sample(which(!fonts$italic & fonts$weight > 'normal'), 1)
-#' italic <- sample(which(fonts$italic & fonts$weight <= 'normal'), 1)
-#' bolditalic <- sample(which(fonts$italic & fonts$weight > 'normal'), 1)
+#' random_fonts <- sample(nrow(fonts), 4)
 #' register_font(
 #'   'random',
-#'   plain = list(fonts$path[plain], fonts$index[plain]),
-#'   bold = list(fonts$path[bold], fonts$index[bold]),
-#'   italic = list(fonts$path[italic], fonts$index[italic]),
-#'   bolditalic = list(fonts$path[bolditalic], fonts$index[bolditalic])
+#'   plain = list(fonts$path[random_fonts[1]], fonts$index[random_fonts[1]]),
+#'   bold = list(fonts$path[random_fonts[2]], fonts$index[random_fonts[2]]),
+#'   italic = list(fonts$path[random_fonts[3]], fonts$index[random_fonts[3]]),
+#'   bolditalic = list(fonts$path[random_fonts[4]], fonts$index[random_fonts[4]])
 #' )
 #'
 #' # Look at your creation
@@ -57,21 +54,42 @@
 #' # Reset
 #' clear_registry()
 #'
-register_font <- function(name, plain, bold = plain, italic = plain, bolditalic = plain, features = font_feature()) {
+register_font <- function(
+  name,
+  plain,
+  bold = plain,
+  italic = plain,
+  bolditalic = plain,
+  features = font_feature()
+) {
   if (name %in% system_fonts()$family) {
-    stop("A system font with that family name already exists", call. = FALSE)
+    stop("A system font called `", name, "` already exists", call. = FALSE)
+  }
+  if (is.character(plain)) {
+    plain <- list(plain, 0)
+  }
+  if (is.character(bold)) {
+    bold <- list(bold, 0)
+  }
+  if (is.character(italic)) {
+    italic <- list(italic, 0)
+  }
+  if (is.character(bolditalic)) {
+    bolditalic <- list(bolditalic, 0)
   }
-  if (is.character(plain)) plain <- list(plain, 0)
-  if (is.character(bold)) bold <- list(bold, 0)
-  if (is.character(italic)) italic <- list(italic, 0)
-  if (is.character(bolditalic)) bolditalic <- list(bolditalic, 0)
   files <- c(plain[[1]], bold[[1]], italic[[1]], bolditalic[[1]])
   indices <- c(plain[[2]], bold[[2]], italic[[2]], bolditalic[[2]])
   if (!all(file.exists(files))) {
-    stop("reference to non-existing font file", call. = FALSE)
+    stop(name, " refers to non-existing font file(s)", call. = FALSE)
   }
 
-  register_font_c(as.character(name), as.character(files), as.integer(indices), features[[1]], features[[2]])
+  register_font_c(
+    as.character(name),
+    as.character(files),
+    as.integer(indices),
+    features[[1]],
+    features[[2]]
+  )
 }
 #' @rdname register_font
 #' @export
@@ -126,20 +144,41 @@ clear_registry <- function() {
 #' # clean up
 #' clear_registry()
 #'
-register_variant <- function(name, family, weight = NULL, width = NULL, features = font_feature()) {
+register_variant <- function(
+  name,
+  family,
+  weight = NULL,
+  width = NULL,
+  features = font_feature()
+) {
   sys_fonts <- system_fonts()
-  sys_fonts <- sys_fonts[grepl(tolower(family), tolower(sys_fonts$family)), , drop = FALSE]
+  sys_fonts <- sys_fonts[
+    grepl(tolower(family), tolower(sys_fonts$family)),
+    ,
+    drop = FALSE
+  ]
   if (!is.null(width)) {
     sys_fonts <- sys_fonts[sys_fonts$width == tolower(width), , drop = FALSE]
   }
   if (!is.null(weight)) {
-    sys_fonts <- sys_fonts[sys_fonts$weight %in% tolower(weight), , drop = FALSE]
+    sys_fonts <- sys_fonts[
+      sys_fonts$weight %in% tolower(weight),
+      ,
+      drop = FALSE
+    ]
   }
   if (nrow(sys_fonts) == 0) {
-    stop("No font with the given family name and weight/width settings available", call. = FALSE)
+    stop(
+      "No font with the given family name and weight/width settings available",
+      call. = FALSE
+    )
   }
   if (any(tolower(family) == tolower(sys_fonts$family))) {
-    sys_fonts <- sys_fonts[tolower(family) == tolower(sys_fonts$family), , drop = FALSE]
+    sys_fonts <- sys_fonts[
+      tolower(family) == tolower(sys_fonts$family),
+      ,
+      drop = FALSE
+    ]
   }
   if (is.null(width)) {
     width <- sys_fonts$width[which.min(abs(as.integer(sys_fonts$width) - 5))]
@@ -150,14 +189,46 @@ register_variant <- function(name, family, weight = NULL, width = NULL, features
     bold <- which.min(abs(as.integer(sys_fonts$weight) - 7))
     weight <- sys_fonts$weight[unique(c(normal, bold))]
   }
-  plain <- sys_fonts[which(sys_fonts$weight == weight[1] & !sys_fonts$italic), , drop = FALSE]
-  bold <- if (length(weight) == 2) sys_fonts[which(sys_fonts$weight == weight[2] & !sys_fonts$italic), , drop = FALSE] else plain
-  italic <- sys_fonts[which(sys_fonts$weight == weight[1] & sys_fonts$italic), , drop = FALSE]
-  bolditalic <- if (length(weight) == 2) sys_fonts[which(sys_fonts$weight == weight[2] & sys_fonts$italic), , drop = FALSE] else italic
-  if (nrow(plain) == 0) plain <- italic
-  if (nrow(bold) == 0) bold <- plain
-  if (nrow(italic) == 0) italic <- plain
-  if (nrow(bolditalic) == 0) bolditalic <- if (length(weight) == 2) bold else italic
+  plain <- sys_fonts[
+    which(sys_fonts$weight == weight[1] & !sys_fonts$italic),
+    ,
+    drop = FALSE
+  ]
+  bold <- if (length(weight) == 2) {
+    sys_fonts[
+      which(sys_fonts$weight == weight[2] & !sys_fonts$italic),
+      ,
+      drop = FALSE
+    ]
+  } else {
+    plain
+  }
+  italic <- sys_fonts[
+    which(sys_fonts$weight == weight[1] & sys_fonts$italic),
+    ,
+    drop = FALSE
+  ]
+  bolditalic <- if (length(weight) == 2) {
+    sys_fonts[
+      which(sys_fonts$weight == weight[2] & sys_fonts$italic),
+      ,
+      drop = FALSE
+    ]
+  } else {
+    italic
+  }
+  if (nrow(plain) == 0) {
+    plain <- italic
+  }
+  if (nrow(bold) == 0) {
+    bold <- plain
+  }
+  if (nrow(italic) == 0) {
+    italic <- plain
+  }
+  if (nrow(bolditalic) == 0) {
+    bolditalic <- if (length(weight) == 2) bold else italic
+  }
   register_font(
     name,
     as.list(plain[1, c('path', 'index')]),
@@ -174,7 +245,7 @@ register_variant <- function(name, family, weight = NULL, width = NULL, features
 #' installed on the OS you are executing the code on. However, you may want to
 #' access fonts without doing a full installation, either because you want your
 #' project to be reproducible on all systems, because you don't have
-#' administrator priviliges on the system, or for a different reason entirely.
+#' administrator privileges on the system, or for a different reason entirely.
 #' `add_fonts()` provide a way to side load font files so that they are found
 #' during font matching. The function differs from [register_font()] and
 #' [register_variant()] in that they add the font file as-is using the family
@@ -184,7 +255,7 @@ register_variant <- function(name, family, weight = NULL, width = NULL, features
 #'
 #' @inheritSection match_fonts Font matching
 #'
-#' @param files A character vector of font file paths to add
+#' @param files A character vector of font file paths or urls to add
 #'
 #' @return This function is called for its sideeffects
 #'
@@ -202,11 +273,29 @@ add_fonts <- function(files) {
   if (!is.character(files)) {
     stop("`files` must be a character vector")
   }
-  if (!all(file.exists(files))) {
-    stop("all elements in `files` must be paths to existing files")
+  urls <- grepl("^https?://", files)
+  if (any(urls)) {
+    dest <- vapply(which(urls), function(i) tempfile(), character(1))
+    success <- utils::download.file(files[urls], dest, method = "libcurl")
+    if (success != 0) {
+      stop(
+        "Download of font files failed with the libcurl error ",
+        success,
+        call. = FALSE
+      )
+    }
+    files[urls] <- dest
+  }
+  do_exist <- file.exists(files)
+  if (!all(do_exist)) {
+    warning(
+      "all elements in `files` must be paths to existing files. Ignoring ",
+      paste0(files[!do_exist], collapse = ", ")
+    )
+    files <- files[do_exist]
   }
   if (length(files) > 0) {
-    add_local_fonts(vapply(files, tools::file_path_as_absolute, character(1)))
+    add_local_fonts(vapply(files, normalizePath, character(1)))
   }
   invisible(NULL)
 }
@@ -216,7 +305,12 @@ add_fonts <- function(files) {
 #'
 scan_local_fonts <- function() {
   files <- unique(c(
-    list.files("./fonts", all.files = TRUE, full.names = TRUE, recursive = TRUE),
+    list.files(
+      "./fonts",
+      all.files = TRUE,
+      full.names = TRUE,
+      recursive = TRUE
+    ),
     list.files("~/fonts", all.files = TRUE, full.names = TRUE, recursive = TRUE)
   ))
   add_fonts(files)
@@ -225,6 +319,8 @@ scan_local_fonts <- function() {
 #' @export
 #'
 clear_local_fonts <- function() {
-  message("Run `scan_local_fonts()` in order to re-add automatically added fonts")
+  message(
+    "Run `scan_local_fonts()` in order to re-add automatically added fonts"
+  )
   clear_local_fonts_c()
 }
